/**********************************************************************
 *
 *  geotiff.h - Public interface for Geotiff tag parsing.
 *
 *   Written By: Niles D. Ritter
 *
 *  copyright (c) 1995   Niles D. Ritter
 *
 *  Permission granted to use this software, so long as this copyright
 *  notice accompanies any products derived therefrom.
 **********************************************************************/

#ifndef __geotiff_h_
#define __geotiff_h_

/**
 * \file geotiff.h
 *
 * Primary libgeotiff include file.
 *
 * This is the defacto registry for valid GEOTIFF GeoKeys
 * and their associated symbolic values. This is also the only file
 * of the GeoTIFF library which needs to be included in client source
 * code.
 */

/* This Version code should only change if a drastic
 * alteration is made to the GeoTIFF key structure. Readers
 * encountering a larger value should give up gracefully.
 */
#define GvCurrentVersion   1

#define LIBGEOTIFF_VERSION 1250

#include "geo_config.h"
#include "geokeys.h"

/**********************************************************************
 * Do we want to build as a DLL on windows?
 **********************************************************************/
#if !defined(LIBGEOTIFF_DLL_EXPORTED)
#  if (defined(_WIN32) || defined(__CYGWIN__)) && defined(BUILDING_LIBGEOTIFF) && defined(DLL_EXPORT)
#    define LIBGEOTIFF_DLL_EXPORTED	__declspec(dllexport)
#  else
#    define LIBGEOTIFF_DLL_EXPORTED
#  endif
#endif

/**********************************************************************
 *
 *                 Public Structures & Definitions
 *
 **********************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif

typedef struct gtiff GTIF;   /* struct gtiff is private */
typedef struct _TIFFMethod TIFFMethod;
typedef unsigned short tifftag_t;
typedef unsigned short geocode_t;
typedef int (*GTIFPrintMethod)(char *string, void *aux);
typedef int (*GTIFReadMethod)(char *string, void *aux);

typedef enum {
   TYPE_BYTE=1,
   TYPE_SHORT=2,
   TYPE_LONG=3,
   TYPE_RATIONAL=4,
   TYPE_ASCII=5,
   TYPE_FLOAT=6,
   TYPE_DOUBLE=7,
   TYPE_SBYTE=8,
   TYPE_SSHORT=9,
   TYPE_SLONG=10,
   TYPE_UNKNOWN=11
} tagtype_t;


/**********************************************************************
 *
 *                 Public Function Declarations
 *
 **********************************************************************/

/* TIFF-level interface */
GTIF LIBGEOTIFF_DLL_EXPORTED *GTIFNew(void *tif);
GTIF LIBGEOTIFF_DLL_EXPORTED *GTIFNewSimpleTags(void *tif);
GTIF LIBGEOTIFF_DLL_EXPORTED *GTIFNewWithMethods(void *tif, TIFFMethod*);
void LIBGEOTIFF_DLL_EXPORTED  GTIFFree(GTIF *gtif);
int  LIBGEOTIFF_DLL_EXPORTED  GTIFWriteKeys(GTIF *gtif);
void LIBGEOTIFF_DLL_EXPORTED  GTIFDirectoryInfo(GTIF *gtif, int *versions, int *keycount);

/* GeoKey Access */
int  LIBGEOTIFF_DLL_EXPORTED  GTIFKeyInfo(GTIF *gtif, geokey_t key, int *size, tagtype_t* type);
int  LIBGEOTIFF_DLL_EXPORTED  GTIFKeyGet(GTIF *gtif, geokey_t key, void *val, int index,
                         int count);
int  LIBGEOTIFF_DLL_EXPORTED  GTIFKeySet(GTIF *gtif, geokey_t keyID, tagtype_t type,
                         int count,...);

/* Metadata Import-Export utilities */
void  LIBGEOTIFF_DLL_EXPORTED  GTIFPrint(GTIF *gtif, GTIFPrintMethod print, void *aux);
int   LIBGEOTIFF_DLL_EXPORTED  GTIFImport(GTIF *gtif, GTIFReadMethod scan, void *aux);
char  LIBGEOTIFF_DLL_EXPORTED *GTIFKeyName(geokey_t key);
char  LIBGEOTIFF_DLL_EXPORTED *GTIFValueName(geokey_t key,int value);
char  LIBGEOTIFF_DLL_EXPORTED *GTIFTypeName(tagtype_t type);
char  LIBGEOTIFF_DLL_EXPORTED *GTIFTagName(int tag);
int   LIBGEOTIFF_DLL_EXPORTED  GTIFKeyCode(char * key);
int   LIBGEOTIFF_DLL_EXPORTED  GTIFValueCode(geokey_t key,char *value);
int   LIBGEOTIFF_DLL_EXPORTED  GTIFTypeCode(char *type);
int   LIBGEOTIFF_DLL_EXPORTED  GTIFTagCode(char *tag);

/* Translation between image/PCS space */

int LIBGEOTIFF_DLL_EXPORTED    GTIFImageToPCS( GTIF *gtif, double *x, double *y );
int LIBGEOTIFF_DLL_EXPORTED    GTIFPCSToImage( GTIF *gtif, double *x, double *y );

#if defined(__cplusplus)
}
#endif

#endif /* __geotiff_h_ */
